var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { createContext, memo, useCallback, useContext, useEffect, useMemo, useRef, useState } from 'react';
import { useDeep } from '@deep-foundation/deeplinks/imports/client';
import { gql } from '@apollo/client/index';
import $ from 'jquery';
import isEqual from 'lodash/isEqual';
import React from 'react';
import { WatchLink } from './react-handler';
import { GoProvider, useGoCore } from './go';
import { useHandlersContext, usePreload } from './hooks';
export class CatchErrors extends React.Component {
    constructor(props) {
        super(props);
        this.errorRenderer = (error, reset, catcher) => React.createElement(React.Fragment, null, "error native");
        this.state = { error: undefined };
        this.reset = () => {
            var _a, _b;
            this.setState({ error: undefined });
            ((_a = this === null || this === void 0 ? void 0 : this.props) === null || _a === void 0 ? void 0 : _a.reset) && ((_b = this === null || this === void 0 ? void 0 : this.props) === null || _b === void 0 ? void 0 : _b.reset());
        };
    }
    static getDerivedStateFromError(error) {
        console.log('getDerivedStateFromError', error);
        return { error };
    }
    componentDidCatch(error, errorInfo) {
        var err = error.constructor('Error in Evaled Script: ' + error.message);
        err.lineNumber = error.lineNumber - err.lineNumber + 3;
        console.log('componentDidCatch', error, errorInfo);
    }
    componentDidMounted() {
        var _a, _b;
        ((_a = this === null || this === void 0 ? void 0 : this.props) === null || _a === void 0 ? void 0 : _a.onMounted) && ((_b = this === null || this === void 0 ? void 0 : this.props) === null || _b === void 0 ? void 0 : _b.onMounted((error) => this.setState({ error: error })));
    }
    render() {
        var _a, _b;
        const error = this.props.error || this.state.error;
        if (error) {
            return ((_a = this === null || this === void 0 ? void 0 : this.props) === null || _a === void 0 ? void 0 : _a.errorRenderer) ? (_b = this === null || this === void 0 ? void 0 : this.props) === null || _b === void 0 ? void 0 : _b.errorRenderer(error, this.reset, this) : this === null || this === void 0 ? void 0 : this.errorRenderer(error, this.reset, this);
        }
        return this.props.children;
    }
}
export function evalClientHandler(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const { value, deep, input = {}, } = options;
        try {
            const evalResult = eval(value);
            if (typeof evalResult === 'function') {
                return {
                    data: yield evalResult(Object.assign({ deep, gql }, input)),
                };
            }
            else {
                return {
                    error: new Error(`Client handler must return a function, got ${typeof evalResult}`),
                };
            }
        }
        catch (error) {
            console.error('evalClientHandler', 'error', error);
            return { error };
        }
        return {};
    });
}
export const HandlersGoContext = createContext(undefined);
HandlersGoContext.displayName = 'HandlersGoContext';
export function useHandlersGo() { return useContext(HandlersGoContext); }
export const ClientHandler = memo(function ClientHandler(_props) {
    var _a;
    const deep = useDeep();
    const ml = deep.minilinks;
    const go = useGoCore();
    const goHandler = useHandlersGo();
    const [isPreloaded, repreload] = usePreload();
    const { ErrorComponent, UnhandledComponent, } = _props;
    const _b = useClientHandler(_props), { data: Component, file, sync, linkId, handlerId, context = [], onClose, outerError, error, require, errored, erroredResetRef, setErrorRef, reloadHandler } = _b, props = __rest(_b, ["data", "file", "sync", "linkId", "handlerId", "context", "onClose", "outerError", "error", "require", "errored", "erroredResetRef", "setErrorRef", "reloadHandler"]);
    const [link] = deep.useLinks(linkId);
    return (React.createElement(React.Fragment, null,
        React.createElement(GoProvider, { context: HandlersGoContext, linkId: handlerId },
            !!file && !!sync && React.createElement(WatchLink, { link: file }),
            React.createElement(CatchErrors, { error: errored || outerError, errorRenderer: (error, reset) => {
                    erroredResetRef.current = () => __awaiter(this, void 0, void 0, function* () {
                        reloadHandler();
                        reset();
                    });
                    return React.createElement("div", null,
                        React.createElement(ErrorComponent, Object.assign({ UnhandledComponent: UnhandledComponent, ErrorComponent: ErrorComponent, Component: Component, link: link, file: file }, props, { handlerId: handlerId, linkId: linkId, error: error, reset: erroredResetRef.current, goHandler: goHandler, go: go })));
                }, onMounted: (setError) => setErrorRef.current = setError }, Component ? React.createElement(React.Fragment, null, [React.createElement(ClientHandlerRenderer, Object.assign({ key: (_a = file === null || file === void 0 ? void 0 : file.value) === null || _a === void 0 ? void 0 : _a.value, Component: Component, link: link }, props, { linkId: linkId, handlerId: handlerId, goHandler: goHandler, go: go }))]) : React.createElement(React.Fragment, null,
                React.createElement(UnhandledComponent, Object.assign({ UnhandledComponent: UnhandledComponent, ErrorComponent: ErrorComponent, Component: Component, file: file, link: link }, props, { linkId: linkId, handlerId: handlerId, goHandler: goHandler, go: go })))))));
});
export const HandlerConfigContext = createContext({ sync: false, setSync: (value) => { } });
const _client_handlers_memory = {};
export function useClientHandler(_props) {
    var _a, _b;
    const { sync: __sync = false } = useContext(HandlerConfigContext);
    const { link: _link, linkId: _linkId, handlerId, context = [], onClose, error: outerError, require, sync: _sync = __sync } = _props, props = __rest(_props, ["link", "linkId", "handlerId", "context", "onClose", "error", "require", "sync"]);
    const deep = useDeep();
    const go = useGoCore();
    const goHandler = useHandlersGo();
    const ml = deep === null || deep === void 0 ? void 0 : deep.minilinks;
    const { handler: hid, reloadHandler, counter } = useFindClientHandler(_props);
    const [{ Component, errored } = {}, setState] = useState({ Component: undefined, errored: undefined });
    const [sync, setSync] = useState(_sync);
    const link = _link || ((_a = ml === null || ml === void 0 ? void 0 : ml.byId) === null || _a === void 0 ? void 0 : _a[_linkId]);
    const linkId = (link === null || link === void 0 ? void 0 : link.id) || _linkId;
    const files = deep.useMinilinksSubscription({
        id: (hid === null || hid === void 0 ? void 0 : hid.dist_id) || 0,
    });
    const file = files === null || files === void 0 ? void 0 : files[0];
    const fileRef = useRef(file);
    fileRef.current = file;
    const lastEvalRef = useRef(0);
    useEffect(() => {
        var _a, _b, _c, _d;
        if (typeof (window) === 'object') {
            window._deep_perception_client_handlers_memory = _client_handlers_memory;
        }
        if (!hid || !(file === null || file === void 0 ? void 0 : file.id))
            return;
        const value = (_b = (_a = deep.minilinks.byId[file === null || file === void 0 ? void 0 : file.id]) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.value;
        if (!value)
            return;
        if (!(_client_handlers_memory === null || _client_handlers_memory === void 0 ? void 0 : _client_handlers_memory[file === null || file === void 0 ? void 0 : file.id]) || ((_c = _client_handlers_memory === null || _client_handlers_memory === void 0 ? void 0 : _client_handlers_memory[file === null || file === void 0 ? void 0 : file.id]) === null || _c === void 0 ? void 0 : _c.value) != ((_d = file === null || file === void 0 ? void 0 : file.value) === null || _d === void 0 ? void 0 : _d.value)) {
            const evalId = ++lastEvalRef.current;
            evalClientHandler({ value, deep, input: { require, Go: go } }).then(({ data, error }) => {
                var _a, _b, _c;
                try {
                    if (evalId === lastEvalRef.current) {
                        if (!error) {
                            _client_handlers_memory[file.id] = {
                                value: (_a = file === null || file === void 0 ? void 0 : file.value) === null || _a === void 0 ? void 0 : _a.value,
                                Component: React.memo(React.forwardRef(data), isEqual),
                            };
                            setState({ Component: _client_handlers_memory[file.id].Component });
                            (erroredResetRef === null || erroredResetRef === void 0 ? void 0 : erroredResetRef.current) && (erroredResetRef === null || erroredResetRef === void 0 ? void 0 : erroredResetRef.current(), erroredResetRef.current = undefined);
                        }
                        else {
                            _client_handlers_memory[file.id] = {
                                value: (_b = file === null || file === void 0 ? void 0 : file.value) === null || _b === void 0 ? void 0 : _b.value,
                                error,
                            };
                            setErrorRef.current && setErrorRef.current(error);
                            setState({ Component: undefined, errored: error });
                        }
                    }
                    else {
                    }
                }
                catch (error) {
                    _client_handlers_memory[file.id] = {
                        value: (_c = file === null || file === void 0 ? void 0 : file.value) === null || _c === void 0 ? void 0 : _c.value,
                        error,
                    };
                    setErrorRef.current && setErrorRef.current(error);
                    setState({ Component: undefined, errored: error });
                }
            });
        }
        else {
            setState({ Component: _client_handlers_memory[file.id].Component });
        }
    }, [(_b = file === null || file === void 0 ? void 0 : file.value) === null || _b === void 0 ? void 0 : _b.value, counter]);
    const erroredResetRef = useRef();
    const setErrorRef = useRef();
    useEffect(() => {
        if (!!errored && !sync)
            setSync(true);
    }, [errored, sync]);
    return Object.assign(Object.assign(Object.assign({}, _props), props), { data: Component, handlerId,
        file,
        sync,
        errored,
        outerError,
        erroredResetRef,
        setErrorRef,
        link,
        linkId,
        require,
        reloadHandler });
}
;
export const ClientHandlerRenderer = React.memo(function ClientHandlerRenderer(_a) {
    var { Component, fillSize = false, onClose } = _a, props = __rest(_a, ["Component", "fillSize", "onClose"]);
    const hgo = useHandlersGo();
    const go = useGoCore();
    const goRef = useRef(go);
    goRef.current = go;
    const hgoRef = useRef(hgo);
    hgoRef.current = hgo;
    useEffect(() => {
        setAttrs(go, hgo);
    }, [go, hgo]);
    const setAttrs = useCallback((go, hgo) => {
        if (!!hgo.ref.current) {
            const el = hgo.ref.current;
            if (el) {
                $(el).addClass("deep-link-id");
                $(el).data('deep-link-id', props.handlerId);
                $(el).data('deep-link-go', go);
                $(el).data('deep-link-handlers-go', hgo);
            }
        }
    }, []);
    useEffect(() => {
        const i = setInterval(() => {
            const go = goRef.current;
            const hgo = hgoRef.current;
            if (!!hgo.ref.current) {
                clearInterval(i);
                setAttrs(go, hgo);
            }
        }, 100);
        return () => clearInterval(i);
    }, []);
    return React.createElement(React.Fragment, null, !!Component && React.createElement(Component, Object.assign({ onClose: onClose, fillSize: fillSize }, props, { ref: hgo.ref })));
});
export function useFindClientHandler({ handlerId, handlerQuery, context = [], }) {
    const deep = useDeep();
    const [asyncHandler, setAsyncHandler] = useState();
    const handlers = useHandlersContext();
    const memoHandler = useMemo(() => {
        return handlerId ? handlers.current.find(h => h.handler_id === handlerId) : undefined;
    }, [handlerId]);
    const [counter, setCounter] = useState(0);
    const reloadHandler = useCallback(() => {
        setCounter(c => c + 1);
    }, []);
    useEffect(() => {
        if (!memoHandler || !!counter)
            (() => __awaiter(this, void 0, void 0, function* () {
                var _a, _b;
                if (asyncHandler)
                    return;
                const handlerIds = handlerQuery ? (_a = (yield deep.select(Object.assign({ type_id: deep.idLocal('@deep-foundation/core', 'Handler') }, handlerQuery)))) === null || _a === void 0 ? void 0 : _a.data : [];
                const _handlerId = handlerId || ((_b = handlerIds[0]) === null || _b === void 0 ? void 0 : _b.id);
                if (deep.isId(_handlerId)) {
                    const handler = yield deep._findHandler({ context, handlerId: _handlerId });
                    if (handler) {
                        setAsyncHandler(handler);
                    }
                }
            }))();
    }, [context, handlerId, handlerQuery, asyncHandler, counter]);
    return { handler: counter && asyncHandler ? asyncHandler : memoHandler, reloadHandler, counter };
}
//# sourceMappingURL=client-handler.js.map