var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { Box, Button, useColorMode, VStack } from '@chakra-ui/react';
import { useDeep } from "@deep-foundation/deeplinks/imports/client";
import React, { useCallback, useMemo, useRef, useState } from 'react';
import ReactCodeMirror from '@uiw/react-codemirror';
import { githubLight, githubDark } from '@uiw/codemirror-theme-github';
import { langs } from '@uiw/codemirror-extensions-langs';
import { history as History } from "@codemirror/commands";
import { keymap } from "@codemirror/view";
import { linter } from "@codemirror/lint";
import { esLint } from "@codemirror/lang-javascript";
import * as eslint from "eslint-linter-browserify";
import globals from "globals";
import isEqual from 'lodash/isEqual';
import { MdSaveAlt } from 'react-icons/md';
import { useResizeDetector } from 'react-resize-detector';
const eslintConfig = {
    languageOptions: {
        globals: Object.assign({}, globals.node),
        parserOptions: {
            ecmaVersion: 2022,
            sourceType: "module",
        },
    },
    rules: {
        semi: ["error", "never"],
    },
};
function usePsudoResize() {
    const ref = useRef();
    return useMemo(() => ({ width: 0, height: 0, ref }), []);
}
export const Editor = React.memo(function Editor(_a) {
    var _b, _c;
    var { refEditor, linkId, subscription = false, value = '', onChange, onSave, onClose, onExit, saveButton = true, minimap = true, lineNumbers, defaultLanguage = "javascript", fillSize = false, onMount, children = null } = _a, props = __rest(_a, ["refEditor", "linkId", "subscription", "value", "onChange", "onSave", "onClose", "onExit", "saveButton", "minimap", "lineNumbers", "defaultLanguage", "fillSize", "onMount", "children"]);
    const deep = useDeep();
    if (!!value && typeof (value) !== 'string')
        throw new Error(`Editor value must be string but ${deep.stringify(value)}`);
    const [startValue, setStartValue] = useState(value);
    const [v, setV] = useState(value);
    const { colorMode } = useColorMode();
    const [lang, setLang] = useState(langs.tsx());
    const _refEditor = useRef();
    const eref = refEditor || _refEditor;
    const useLoad = useMemo(() => {
        return linkId ? deep[subscription ? 'useSubscription' : 'useQuery'] : function useLoad() { return { data: [] }; };
    }, []);
    const { data: [link] } = useLoad({ id: linkId });
    const update = useCallback(() => {
        if (linkId && (v === null || v === void 0 ? void 0 : v.length) > 0)
            deep.value(linkId, v);
        onSave && onSave(v);
    }, [link, v]);
    const customKeymap = useMemo(() => {
        const save = (editor) => {
            update();
            return true;
        };
        return keymap.of([
            { key: "cmd-s", run: save },
            { key: "ctrl-s", run: save },
        ]);
    }, [v]);
    const { width, height, ref } = (fillSize ? useResizeDetector : usePsudoResize)();
    const history = useMemo(() => History(), []);
    const extensions = useMemo(() => [
        lang,
        customKeymap,
        linter(esLint(new eslint.Linter(), eslintConfig)),
        history,
        ...((props === null || props === void 0 ? void 0 : props.extensions) || []),
    ], [customKeymap]);
    const basicSetup = useMemo(() => (Object.assign({ tabSize: 2, lineNumbers, lineWrapping: true }, ((props === null || props === void 0 ? void 0 : props.basicSetup) || {}))), [props === null || props === void 0 ? void 0 : props.basicSetup]);
    const _onChange = useCallback((value, viewUpdate) => {
        setV(value);
        onChange && onChange(value);
    }, [onChange]);
    const other = useMemo(() => ((fillSize ? { height: `${height}px`, width: `${width}px` } : {})), [fillSize, height, width]);
    return React.createElement(Box, Object.assign({}, (fillSize ? { h: '100%', w: '100%' } : {}), { position: 'relative', overflow: 'hidden', ref: fillSize ? ref : undefined }),
        React.createElement(Box, Object.assign({}, (fillSize ? { position: 'absolute', left: '0', top: '0', right: '0', bottom: '0' } : {}), { w: '100%' }), (linkId ? !!link : true) && React.createElement(ReactCodeMirror, Object.assign({ ref: eref, value: (props === null || props === void 0 ? void 0 : props.editable) !== false && !(props === null || props === void 0 ? void 0 : props.readOnly) ? (linkId ? deep.stringify((_b = link === null || link === void 0 ? void 0 : link.value) === null || _b === void 0 ? void 0 : _b.value) || startValue : startValue) || '' : value, theme: colorMode === 'light' ? githubLight : githubDark, extensions: extensions, basicSetup: basicSetup, onChange: _onChange }, other, props))),
        React.createElement(VStack, { position: 'absolute', right: '1em', top: '1em', overflow: 'hidden' },
            children,
            !!saveButton && (props === null || props === void 0 ? void 0 : props.editable) !== false && !(props === null || props === void 0 ? void 0 : props.readOnly) && React.createElement(Button, { w: '3em', h: '3em', transition: 'all 1s ease', position: 'relative', left: !subscription || !isEqual(v, ((_c = link === null || link === void 0 ? void 0 : link.value) === null || _c === void 0 ? void 0 : _c.value) || value) ? '10em' : '0em', boxShadow: 'dark-lg', variant: undefined, onClick: update },
                React.createElement(MdSaveAlt, null))));
}, isEqual);
//# sourceMappingURL=editor.js.map